public inbox for linux-kernel@vger.kernel.org
 help / color / mirror / Atom feed
* [patch] tlb flush_data: replace per_cpu with an array
@ 2009-01-12 21:35 Frederik Deweerdt
  2009-01-12 21:46 ` Ingo Molnar
                   ` (3 more replies)
  0 siblings, 4 replies; 31+ messages in thread
From: Frederik Deweerdt @ 2009-01-12 21:35 UTC (permalink / raw)
  To: mingo; +Cc: andi, tglx, hpa, linux-kernel

Hi,

On x86_64 flush tlb data is stored in per_cpu variables. This is
unnecessary because only the first NUM_INVALIDATE_TLB_VECTORS entries
are accessed.
This patch aims at making the code less confusing (there's nothing
really "per_cpu") by using a plain array. It also would save some memory
on most distros out there (Ubuntu x86_64 has NR_CPUS=64 by default).

Regards,
Frederik

Signed-off-by: Frederik Deweerdt <frederik.deweerdt@xprog.eu>

diff --git a/arch/x86/kernel/tlb_64.c b/arch/x86/kernel/tlb_64.c
index f8be6f1..c177a1f 100644
--- a/arch/x86/kernel/tlb_64.c
+++ b/arch/x86/kernel/tlb_64.c
@@ -33,7 +33,7 @@
  *	To avoid global state use 8 different call vectors.
  *	Each CPU uses a specific vector to trigger flushes on other
  *	CPUs. Depending on the received vector the target CPUs look into
  *	the right per cpu variable for the flush data.
  *
  *	With more than 8 CPUs they are hashed to the 8 available
  *	vectors. The limited global vector space forces us to this right now.
@@ -54,7 +54,7 @@ union smp_flush_state {
 /* State is put into the per CPU data section, but padded
    to a full cache line because other CPUs can access it and we don't
    want false sharing in the per cpu data segment. */
-static DEFINE_PER_CPU(union smp_flush_state, flush_state);
+static union smp_flush_state flush_state[NUM_INVALIDATE_TLB_VECTORS];
 
 /*
  * We cannot call mmdrop() because we are in interrupt context,
@@ -129,7 +129,7 @@ asmlinkage void smp_invalidate_interrupt(struct pt_regs *regs)
 	 * Use that to determine where the sender put the data.
 	 */
 	sender = ~regs->orig_ax - INVALIDATE_TLB_VECTOR_START;
-	f = &per_cpu(flush_state, sender);
+	f = &flush_state[sender];
 
 	if (!cpu_isset(cpu, f->flush_cpumask))
 		goto out;
@@ -169,7 +169,7 @@ void native_flush_tlb_others(const cpumask_t *cpumaskp, struct mm_struct *mm,
 
 	/* Caller has disabled preemption */
 	sender = smp_processor_id() % NUM_INVALIDATE_TLB_VECTORS;
-	f = &per_cpu(flush_state, sender);
+	f = &flush_state[sender];
 
 	/*
 	 * Could avoid this lock when
@@ -205,8 +205,8 @@ static int __cpuinit init_smp_flush(void)
 {
 	int i;
 
-	for_each_possible_cpu(i)
-		spin_lock_init(&per_cpu(flush_state, i).tlbstate_lock);
+	for (i = 0; i < ARRAY_SIZE(flush_state); i++)
+		spin_lock_init(&flush_state[i].tlbstate_lock);
 
 	return 0;
 }

^ permalink raw reply related	[flat|nested] 31+ messages in thread

end of thread, other threads:[~2009-01-15  7:15 UTC | newest]

Thread overview: 31+ messages (download: mbox.gz follow: Atom feed
-- links below jump to the message on this page --
@@ -33,7 +33,7 @@
  *	Each CPU uses a specific vector to trigger flushes on other
  *	CPUs. Depending on the received vector the target CPUs look into
- *	the right per cpu variable for the flush data.
+ *	the right array slot for the flush data.
  *
  *	With more than 8 CPUs they are hashed to the 8 available
2009-01-12 21:35 [patch] tlb flush_data: replace per_cpu with an array Frederik Deweerdt
2009-01-12 21:46 ` Ingo Molnar
2009-01-12 21:57 ` Andi Kleen
2009-01-12 22:10   ` Frederik Deweerdt
2009-01-12 22:32     ` Andi Kleen
2009-01-12 22:50       ` Ingo Molnar
2009-01-12 22:40   ` Ingo Molnar
2009-01-12 22:59     ` H. Peter Anvin
2009-01-13  2:43     ` Andi Kleen
2009-01-13 12:28       ` Ingo Molnar
2009-01-12 22:34 ` Ravikiran G Thirumalai
2009-01-12 23:00   ` Ingo Molnar
2009-01-12 23:09     ` Ingo Molnar
2009-01-13  2:14     ` Ravikiran G Thirumalai
2009-01-13 12:00     ` Peter Zijlstra
2009-01-13 12:33       ` Ingo Molnar
2009-01-13 18:13         ` Ravikiran G Thirumalai
2009-01-13 18:34           ` Ingo Molnar
2009-01-13 18:42             ` Ravikiran G Thirumalai
2009-01-14  7:31               ` Ingo Molnar
2009-01-15  7:15                 ` Ravikiran G Thirumalai
2009-01-14  9:08       ` Andi Kleen
2009-01-14 14:41         ` Frederik Deweerdt
2009-01-14 15:20           ` Andi Kleen
2009-01-14 15:10             ` Frederik Deweerdt
2009-01-12 22:54 ` Mike Travis
2009-01-12 23:51   ` Frederik Deweerdt
2009-01-12 23:57     ` Mike Travis
2009-01-13  0:01       ` Ingo Molnar
2009-01-13  3:36         ` Andi Kleen
2009-01-13 12:14           ` Ingo Molnar

This is a public inbox, see mirroring instructions
for how to clone and mirror all data and code used for this inbox